/**
 * Rebar — Reading Progress Bar for WordPress.
 * Exclusively on Envato Market: https://1.envato.market/rebar
 *
 * @encoding        UTF-8
 * @version         2.0.3
 * @copyright       Copyright (C) 2018 - 2020 Merkulove ( https://merkulov.design/ ). All rights reserved.
 * @license         Envato License https://1.envato.market/KYbje
 * @contributors    Alexander Khmelnitskiy (info@alexander.khmelnitskiy.ua), Dmitry Merkulov (dmitry@merkulov.design)
 * @support         help@merkulov.design
 **/

/**
 * scrollProgress - v2.1.2
 * https://github.com/jeremenichelli/scrollProgress
 * 2014 (c) Jeremias Menichelli - MIT License
 **/

( function( root, factory ) {
    'use strict';

    if ( typeof define === 'function' && define.amd ) {
        define( function () {
            return factory( root );
        } );
    } else if ( typeof exports === 'object' ) {
        module.exports = factory;
    } else {
        root.scrollProgress = factory( root );
    }
} ) ( this, function () {
    'use strict';

    let body = document.body,
        progress = 0,
        isSet = false,
        progressWrapper,
        progressElement,
        endPoint,

        // default configuration object
        config = {
            color: '#0274e6',
            styles: true,
            prefix: 'mdp-rebar',
            events: true,
            shadow: false
        };

    /**
     * Create DOM elements which graphically represent the progress
     * @method _createElements
     **/
    let _createElements = function () {

        progressWrapper = document.createElement('div');
        progressElement = document.createElement('div');

        progressWrapper.id = config.prefix + '-wrapper';

        /** Add Position and style classes to wrapper. */
         progressWrapper.classList.add( ...['rebar-' + mdpRebarWP.position, mdpRebarWP.style] );

        progressElement.id = config.prefix + '-element';

        progressWrapper.appendChild(progressElement);
        document.body.appendChild(progressWrapper);

    };

    /**
     * Replaces configuration values with custom ones
     * @method _setConfigObject
     * @param {object} obj - object containing custom options
     **/
    let _setConfigObject = function (obj) {
        // override with custom attributes
        if (typeof obj === 'object') {
            for (var key in config) {
                if (typeof obj[key] !== 'undefined') {
                    config[key] = obj[key];
                }
            }
        }
    };

    /**
     * Set styles on DOM elements
     * @method _setElementsStyles
     */
    let _setElementsStyles = function () {
        // setting progress to zero and wrapper to full width
        progressElement.style.width = '0';
        progressWrapper.style.width = '100%';

        // set styles only if
        // settings is true
        if (config.styles) {

            // progress element
            progressElement.style.backgroundColor = config.color;
            if (config.shadow) {
                progressElement.style.boxShadow = '0 0 5px ' + config.color;
            }

            // progress wrapper
            progressWrapper.style.position = 'fixed';
            progressWrapper.style.left = '0';

        }
    };

    /**
     * Main function which sets all variables and bind events if needed
     * @method _set
     * @param {object} custom - object containing custom options
     */
    let _set = function (custom) {
        // set only once
        if ( ! isSet) {
            if (custom) {
                _setConfigObject(custom);
            }
            _createElements();
            _setElementsStyles();

            // set initial metrics
            _setMetrics();

            // bind events only if
            // settings is true
            if (config.events) {
                window.onscroll = _setProgress;
                window.onresize = _setMetrics;
            }

            isSet = true;
        } else {
            throw new Error('scrollProgress has already been set!');
        }
    };

    /**
     * Calculates how much user has scrolled
     * @method _setProgress
     **/
    let _setProgress = function () {

        if ( mdpRebarWP.contentSelector && document.querySelector( mdpRebarWP.contentSelector ) !== null ) {

            /** setProgress for selector */

            let scrolled = window.pageYOffset || (document.documentElement || document.body.parentNode || document.body).scrollTop;
            let windowHeight = window.innerHeight || document.documentElement.clientHeight;

            const selector = document.querySelector( mdpRebarWP.contentSelector );
            let selectorOffsetY = selector.getBoundingClientRect().top - document.body.getBoundingClientRect().top;
            let selectorBegin = scrolled - ( selectorOffsetY - windowHeight );
            let selectorEnd = scrolled - ( selectorOffsetY + selector.getBoundingClientRect().height - windowHeight );

            // Selector in the first screen
            if ( selectorOffsetY < windowHeight ) {
                selectorBegin = scrolled;
            }

            try {

                if ( selectorBegin >= 0 && selectorEnd <= 0 ) {

                    progress = 100 * selectorBegin / selector.getBoundingClientRect().height;

                } else {

                    if ( selectorBegin < 0 ) {
                        progress = 0;
                    } else if ( selectorEnd > 100 ) {
                        progress = 100;
                    }

                }

                progressElement.style.width = progress + '%';

            } catch (e) {

                console.error(e);

            }

        } else {

            /** setProgress for whole page */

            try {

                let y = window.scrollY || window.pageYOffset || document.documentElement.scrollTop;
                endPoint = _getEndPoint();
                progress = y / endPoint * 100;
                progressElement.style.width = progress + '%';

            } catch (e) {

                console.error(e);

            }

        }

    };

    /**
     * Updates the document's height and adjusts the progress bar
     * @method _setMetrics
     */
    let _setMetrics = function () {
        endPoint = _getEndPoint();
        _setProgress();
    };

    /**
     * Returns how much the user can scroll in the document
     * @method _getEndPoint
     */
    let _getEndPoint = function () {
        let res = document.body.scrollHeight - (window.innerHeight || document.documentElement.clientHeight);;
        return res;
    };

    return {
        set: _set,
        trigger: _setProgress,
        update: _setMetrics
    };
});

/** Initialize Reading Progress Bar */
window.onload = function () {

    if ( mdpRebarWP.color ) {
        scrollProgress.set({
            color: mdpRebarWP.color,
            shadow: mdpRebarWP.shadow
        } );
    }

    window.onresize = scrollProgress.update;
    window.onscroll = scrollProgress.trigger;

};
