<?php
/**
 * Plugin Name: ReBar
 * Plugin URI: https://1.envato.market/rebar
 * Description: Reading Progress Bar for WordPress.
 * Author: Merkulove
 * Version: 2.0.3
 * Author URI: https://1.envato.market/cc-merkulove
 * Requires PHP: 5.6
 * Requires at least: 3.0
 * Tested up to: 5.4
 **/

/**
 * Rebar — Reading Progress Bar for WordPress.
 * Exclusively on Envato Market: https://1.envato.market/rebar
 *
 * @encoding        UTF-8
 * @version         2.0.3
 * @copyright       Copyright (C) 2018 - 2020 Merkulove ( https://merkulov.design/ ). All rights reserved.
 * @license         Envato License https://1.envato.market/KYbje
 * @contributors    Alexander Khmelnitskiy (info@alexander.khmelnitskiy.ua), Dmitry Merkulov (dmitry@merkulov.design)
 * @support         help@merkulov.design
 **/

namespace Merkulove;

use Merkulove\ReBar\AssignmentsTab;
use Merkulove\ReBar\EnvatoItem;
use Merkulove\ReBar\PluginHelper;
use Merkulove\ReBar\PluginUpdater;
use Merkulove\ReBar\Settings;

/** Exit if accessed directly. */
if ( ! defined( 'ABSPATH' ) ) {
    header( 'Status: 403 Forbidden' );
    header( 'HTTP/1.1 403 Forbidden' );
    exit;
}

/** Include plugin autoloader for additional classes. */
require __DIR__ . '/src/autoload.php';

/**
 * Core class used to implement a Reading Progress Bar plugin.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * @since 1.0.0
 **/
class ReBar {

	/**
	 * Plugin version.
	 *
	 * @string version
	 * @since 2.0.0
	 **/
	public static $version = '';

	/**
	 * Use minified libraries if SCRIPT_DEBUG is turned off.
	 *
	 * @since 1.0.0
	 **/
	public static $suffix = '';

	/**
	 * URL (with trailing slash) to plugin folder.
	 *
	 * @var string
	 * @since 1.0.0
	 **/
	public static $url = '';

	/**
	 * PATH to plugin folder.
	 *
	 * @var string
	 * @since 1.0.0
	 **/
	public static $path = '';

	/**
	 * Plugin base name.
	 *
	 * @var string
	 * @since 1.0.0
	 **/
	public static $basename = '';

	/**
	 * The one true ReBar.
	 *
	 * @var ReBar
	 * @since 1.0.0
	 **/
	private static $instance;

    /**
     * Sets up a new Reading Progress Bar instance.
     *
     * @since 1.0.0
     * @access public
     */
    public function __construct() {

	    /** Initialize main variables. */
	    $this->initialization();

	    /** Define admin hooks. */
	    $this->admin_hooks();

	    /** Define public hooks. */
	    $this->public_hooks();

	    /** Define hooks that runs on both the front-end as well as the dashboard. */
	    $this->both_hooks();

    }

	/**
	 * Define hooks that runs on both the front-end as well as the dashboard.
	 *
	 * @since 2.0.0
	 * @access private
	 * @return void
	 **/
	private function both_hooks() {

		/** Load translation. */
		add_action( 'plugins_loaded', [$this, 'load_textdomain'] );

	}

	/**
	 * Loads the Plugin translated strings.
	 *
	 * @since 2.0.0
	 * @access public
	 **/
	public function load_textdomain() {

		load_plugin_textdomain( 'rebar', false, dirname( plugin_basename( __FILE__ ) ) . '/languages/' );

	}

	/**
	 * Register all of the hooks related to the public-facing functionality.
	 *
	 * @since 2.0.0
	 * @access private
	 * @return void
	 **/
	private function public_hooks() {

		/** Load CSS for Frontend Area. */
		add_action( 'wp_enqueue_scripts', [$this, 'styles'] ); // CSS.

		/** Load JavaScript for Frontend Area. */
		add_action( 'wp_enqueue_scripts', [$this, 'scripts'] ); // JS.

	}

	/**
	 * Add JavaScript for the public-facing side of the site.
	 *
	 * @return void
	 * @since 2.0.0
	 **/
	public function scripts() {

		/** Checks if plugin should work on this page. */
		if ( ! AssignmentsTab::get_instance()->display() ) { return; }

		/** Enqueue front-end scripts */
		wp_enqueue_script( 'mdp-rebar', self::$url . 'js/rebar' . self::$suffix . '.js', array(), self::$version, true );

		/** Pass variables to frontend. */
		$options = Settings::get_instance()->options;
		wp_localize_script( 'mdp-rebar', 'mdpRebarWP', [
			'color'             => $options['color'],
			'shadow'            => ( 'on' === $options['shadow'] ) ? true : false,
			'position'          => $options['position'],
			'height'            => $options['height'],
			'style'             => $options['style'],
			'contentSelector'   => $options['content_selector'],
		] );

	}

	/**
	 * Add CSS for the public-facing side of the site.
	 *
	 * @return void
	 * @since 2.0.0
	 **/
	public function styles() {

		/** Checks if plugin should work on this page. */
		if ( ! AssignmentsTab::get_instance()->display() ) { return; }

		/** Frontend CSS. */
		wp_enqueue_style( 'mdp-rebar', self::$url . 'css/rebar' . self::$suffix . '.css', [], self::$version );

		$inline_css = $this->get_inline_css();

		/** Add custom CSS. */
		wp_add_inline_style( 'mdp-rebar', $inline_css . Settings::get_instance()->options['custom_css'] );

	}

	/**
	 * Return inline CSS for plugin.
	 *
	 * @since  2.0.0
	 * @access public
	 * @return string
	 **/
	private function get_inline_css() {

		/** Get Plugin Settings. */
		$options = Settings::get_instance()->options;

		/** Height. */
		$height = $options['height'];

		// language=CSS
		return "
			#mdp-rebar-wrapper.rebar-position-left {
				width: calc(100vh + ({$height}px / 2)) !important;
    			left: calc({$height}px / 2) !important;
    			top: calc(-{$height}px / 2);
			}
			
			#mdp-rebar-wrapper.rebar-position-right {
				width: calc(100vh + ({$height}px / 2)) !important;
				left: unset !important;
    			right: calc({$height}px / 2);
    			top: 100vh;
			}

			#mdp-rebar-element {
				height: {$height}px;
			}
        ";

	}

	/**
	 * Register all of the hooks related to the admin area functionality.
	 *
	 * @since 2.0.0
	 * @access private
	 * @return void
	 **/
	private function admin_hooks() {

		/** Add plugin settings page. */
		Settings::get_instance()->add_settings_page();

		/** Load JS and CSS for Backend Area. */
		add_action( 'admin_enqueue_scripts', [ $this, 'admin_styles' ], 100 ); // CSS.
		add_action( 'admin_enqueue_scripts', [ $this, 'admin_scripts' ], 100 ); // JS.

		/** Remove "Thank you for creating with WordPress" and WP version only from plugin settings page. */
		add_action( 'admin_enqueue_scripts', [$this, 'remove_wp_copyrights'] );

		/** Remove all "third-party" notices from plugin settings page. */
		add_action( 'in_admin_header', [$this, 'remove_all_notices'], PHP_INT_MAX );

	}

	/**
	 * Add JS for admin area.
	 *
	 * @return void
	 * @since 2.0.0
	 **/
	public function admin_scripts() {

		/** Get current screen to add scripts on specific pages. */
		$screen = get_current_screen();

		/** Plugin Settings Page. */
		if ( 'toplevel_page_mdp_rebar_settings' === $screen->base ) {

			wp_enqueue_script( 'merkulov-ui', self::$url . 'js/merkulov-ui' . self::$suffix . '.js', [], self::$version, true );
			wp_enqueue_script( 'mdp-rebar-admin', self::$url . 'js/admin' . self::$suffix . '.js', ['jquery'], self::$version, true );
			wp_enqueue_code_editor( array( 'type' => 'text/css' ) ); // Add code editor for Custom CSS.

		}

	}

	/**
	 * Add CSS for admin area.
	 *
	 * @return void
	 * @since 2.0.0
	 **/
	public function admin_styles() {

		/** Get current screen to add styles on specific pages. */
		$screen = get_current_screen();

		/** Plugin Settings Page. */
		if ( 'toplevel_page_mdp_rebar_settings' === $screen->base ) {

			wp_enqueue_style( 'merkulov-ui', self::$url . 'css/merkulov-ui' . self::$suffix . '.css', [], self::$version );
			wp_enqueue_style( 'mdp-rebar-admin', self::$url . 'css/admin' . self::$suffix . '.css', [], self::$version );

			/** Used for preview in admin. */
			wp_enqueue_style( 'mdp-rebar', self::$url . 'css/rebar' . self::$suffix . '.css', array(), self::$version );
			$inline_css = $this->get_inline_css();
			wp_add_inline_style( 'mdp-rebar', $inline_css );

        /** Plugin popup on update. */
		} elseif ( 'plugin-install' === $screen->base ) {

			/** Styles only for our plugin. */
			if ( isset( $_GET['plugin'] ) AND $_GET['plugin'] === 'rebar' ) {
				wp_enqueue_style( 'mdp-rebar-plugin-install', self::$url . 'css/plugin-install' . self::$suffix . '.css', [], self::$version );
			}

		}

	}

	/**
	 * Remove all other notices.
	 *
	 * @since 2.0.0
	 * @access public
	 **/
	public function remove_all_notices() {

		/** Work only on plugin settings page. */
		$screen = get_current_screen();
		if ( $screen->base !== 'toplevel_page_mdp_rebar_settings' ) { return; }

		/** Remove other notices. */
		remove_all_actions( 'admin_notices' );
		remove_all_actions( 'all_admin_notices' );

	}

	/**
	 * Remove "Thank you for creating with WordPress" and WP version only from plugin settings page.
	 *
	 * @since 2.0.0
	 * @access private
	 * @return void
	 **/
	public function remove_wp_copyrights() {

		/** Remove "Thank you for creating with WordPress" and WP version from plugin settings page. */
		$screen = get_current_screen(); // Get current screen.

		/** Plugin Settings Page. */
		if ( 'toplevel_page_mdp_rebar_settings' === $screen->base ) {
			add_filter( 'admin_footer_text', '__return_empty_string', 11 );
			add_filter( 'update_footer', '__return_empty_string', 11 );
		}

	}

	/**
	 * Return plugin version.
	 *
	 * @return string
	 * @since 2.0.0
	 * @access public
	 **/
	public function get_version() {
		return self::$version;
	}

	/**
	 * Initialize main variables.
	 *
	 * @since 2.0.0
	 * @access public
	 **/
	public function initialization() {

		/** Plugin version. */
		if ( ! function_exists('get_plugin_data') ) {
			require_once( ABSPATH . 'wp-admin/includes/plugin.php' );
		}

		$plugin_data = get_plugin_data( __FILE__ );
		self::$version = $plugin_data['Version'];

		/** Gets the plugin URL (with trailing slash). */
		self::$url = plugin_dir_url( __FILE__ );

		/** Gets the plugin PATH. */
		self::$path = plugin_dir_path( __FILE__ );

		/** Use minified libraries if SCRIPT_DEBUG is turned off. */
		self::$suffix = ( defined( 'SCRIPT_DEBUG' ) && SCRIPT_DEBUG ) ? '' : '.min';

		/** Set plugin basename. */
		self::$basename = plugin_basename( __FILE__ );

		/** Initialize plugin settings. */
		Settings::get_instance();

		/** Initialize PluginHelper. */
		PluginHelper::get_instance();

		/** Plugin update mechanism enable only if plugin have Envato ID. */
		$plugin_id = EnvatoItem::get_instance()->get_id();
		if ( (int)$plugin_id > 0 ) {
			PluginUpdater::get_instance();
		}

	}

	/**
	 * Main Rebar Instance.
	 *
	 * Insures that only one instance of Rebar exists in memory at any one time.
	 *
	 * @static
	 * @since 1.0.0
	 * @return ReBar
	 **/
	public static function get_instance() {
		if ( ! isset( self::$instance ) && ! ( self::$instance instanceof ReBar ) ) {
			self::$instance = new ReBar;
		}

		return self::$instance;
	}

} // End of class Rebar.

/** Run Rebar class. */
ReBar::get_instance();
