<?php
/**
 * Rebar — Reading Progress Bar for WordPress.
 * Exclusively on Envato Market: https://1.envato.market/rebar
 *
 * @encoding        UTF-8
 * @version         2.0.3
 * @copyright       Copyright (C) 2018 - 2020 Merkulove ( https://merkulov.design/ ). All rights reserved.
 * @license         Envato License https://1.envato.market/KYbje
 * @contributors    Alexander Khmelnitskiy (info@alexander.khmelnitskiy.ua), Dmitry Merkulov (dmitry@merkulov.design)
 * @support         help@merkulov.design
 **/

namespace Merkulove\ReBar;

use Merkulove\ReBar;
use WP_Query;

/** Exit if accessed directly. */
if ( ! defined( 'ABSPATH' ) ) {
	header( 'Status: 403 Forbidden' );
	header( 'HTTP/1.1 403 Forbidden' );
	exit;
}

/**
 * SINGLETON: Class used to render plugin settings fields.
 *
 * @since 1.0.0
 * @author Alexandr Khmelnytsky (info@alexander.khmelnitskiy.ua)
 **/
final class SettingsFields {

	/**
	 * The one true SettingsFields.
	 *
	 * @var SettingsFields
	 * @since 1.0.0
	 **/
	private static $instance;

	/**
	 * Render Preview box.
	 *
	 * @since 2.0.0
	 * @access public
	 **/
	public static function preview() {

		/** Get plugin settings */
		$options = Settings::get_instance()->options;

		?>
		<div class="mdp-preview">
			<div id="mdp-rebar-wrapper" class="<?php esc_attr_e( $options['style'] ); ?>">
				<div id="mdp-rebar-element" style="background-color: <?php esc_attr_e( $options['color'] ); ?>; height: <?php esc_attr_e( $options['height'] ); ?>; <?php echo ( $options['shadow'] == 'on'  ) ? ' box-shadow: 0 0 5px ' . esc_attr__( $options['color'] ) : ''; ?>"></div>
			</div>
			<div class="mdc-text-field-helper-line">
				<div class="mdc-text-field-helper-text mdc-text-field-helper-text--persistent">
					<?php esc_html_e( 'Change some settings to see how the progress bar will look ', 'rebar' ); ?>
				</div>
			</div>
		</div>
		<?php

	}

	/**
	 * Render Height field.
	 *
	 * @since 2.0.0
	 * @access public
	 **/
	public static function height() {

		/** Height slider. */
		UI::get_instance()->render_slider(
			Settings::get_instance()->options['height'],
			0,
			50,
			1,
			esc_html__( 'Height', 'rebar' ),
			esc_html__( 'Current height: ', 'rebar' ) .
			'<strong>' . Settings::get_instance()->options['height'] . '</strong>' .
			esc_html__( ' px', 'rebar' ),
			[
				'name' => 'mdp_rebar_settings[height]',
				'id' => 'mdp-rebar-settings-height',
				'class' => 'mdc-slider-width'
			]
		);

	}

	/**
	 * Render shadow field.
	 *
	 * @since 2.0.0
	 * @access public
	 **/
	public static function shadow() {

		/** Render Shadow switcher. */
		UI::get_instance()->render_switches(
			Settings::get_instance()->options['shadow'],
			esc_html__('Progress Bar outer shadow', 'rebar' ),
			'',
			[
				'name' => 'mdp_rebar_settings[shadow]',
				'id' => 'mdp-rebar-settings-shadow'
			]
		);

	}

	/**
	 * Render Style field.
	 *
	 * @since 2.0.0
	 * @access public
	 *
	 **/
	public static function style() {

		/** Prepare Style list. */
		$options = [
			'style-default'     => esc_html__( 'Monochrome', 'rebar' ),
			'style-gradient'    => esc_html__( 'Gradient', 'rebar' ),
			'style-rounded'     => esc_html__( 'Rounded', 'rebar' ),
			'style-plastic'     => esc_html__( 'Plastic', 'rebar' ),
			'style-animated'    => esc_html__( 'Animated', 'rebar' ),
			'style-ios7'        => esc_html__( 'Rainbow', 'rebar' ),
			'style-shine'       => esc_html__( 'Shine', 'rebar' ),
			'style-glow'        => esc_html__( 'Glow', 'rebar' )
		];

		/** Render Style dropdown. */
		UI::get_instance()->render_select(
			$options,
			Settings::get_instance()->options['style'], // Selected option.
			esc_html__('Style', 'rebar' ),
			esc_html__( 'Choose the style for the Progress Bar.', 'rebar' ),
			[
				'name' => 'mdp_rebar_settings[style]',
				'id' => 'mdp-rebar-settings-style'
			]
		);

	}

	/**
	 * Render Color field.
	 *
	 * @since 2.0.0
	 * @access public
	 **/
	public static function color() {

		/** Render Color colorpicker. */
		UI::get_instance()->render_colorpicker(
			Settings::get_instance()->options['color'],
			esc_html__( 'Color', 'rebar' ),
			esc_html__( 'Select the Progress Bar background color', 'rebar' ),
			[
				'name'      => 'mdp_rebar_settings[color]',
				'id'        => 'mdp-rebar-settings-color',
				'readonly'  => 'readonly'
			]
		);

	}

	/**
	 * Render Position field.
	 *
	 * @since 2.0.0
	 * @access public
	 *
	 **/
	public static function position() {

		/** Prepare Position list. */
		$options = [
			'position-top'      => esc_html__( 'Top', 'rebar' ),
			'position-bottom'   => esc_html__( 'Bottom', 'rebar' ),
			'position-left'     => esc_html__( 'Left', 'rebar' ),
			'position-right'    => esc_html__( 'Right', 'rebar' )
		];

		/** Render position dropdown. */
		UI::get_instance()->render_select(
			$options,
			Settings::get_instance()->options['position'], // Selected option.
			esc_html__('Position', 'rebar' ),
			esc_html__( 'Choose the position for the Progress Bar.', 'rebar' ),
			[
				'name' => 'mdp_rebar_settings[position]',
				'id' => 'mdp-rebar-settings-position'
			]
		);

	}

	/**
	 * Content Selector field.
	 *
	 * @since 2.0.0
	 * @access public
	 *
	 **/
	public static function content_selector() {

		/** Render Content Selector input. */
		UI::get_instance()->render_input(
			Settings::get_instance()->options['content_selector'], // Selected option.
			esc_html__('Content Selector', 'rebar' ),
			esc_html__( 'Leave blank to use full page height.', 'rebar' ),
			[
				'name' => 'mdp_rebar_settings[content_selector]',
				'id' => 'mdp-rebar-settings-content-selector'
			]
		);

	}

	/**
	 * Render CSS field.
	 *
	 * @since 1.0.0
	 * @access public
	 **/
	public static function custom_css() {
		?>
		<div>
            <label>
                <textarea
                    id="mdp_custom_css_fld"
                    name="mdp_rebar_css_settings[custom_css]"
                    class="mdp_custom_css_fld"><?php echo esc_textarea( Settings::get_instance()->options['custom_css'] ); ?></textarea>
            </label>
			<p class="description"><?php esc_html_e( 'Add custom CSS here.', 'rebar' ); ?></p>
		</div>
		<?php
	}

	/**
	 * Render "SettingsFields Saved" nags.
	 *
	 * @since 1.0.0
	 *
	 * @return void
	 **/
	public static function render_nags() {

		/** Did we try to save settings? */
		if ( ! isset( $_GET['settings-updated'] ) ) { return; }

		/** Are the settings saved successfully? */
		if ( $_GET['settings-updated'] === 'true' ) {

			/** Render "SettingsFields Saved" message. */
			UI::get_instance()->render_snackbar( esc_html__( 'Settings saved!', 'rebar' ) );
		}

		if ( ! isset( $_GET['tab'] ) ) { return; }

		if ( strcmp( $_GET['tab'], "activation" ) == 0 ) {

			if ( PluginActivation::get_instance()->is_activated() ) {

				/** Render "Activation success" message. */
				UI::get_instance()->render_snackbar( esc_html__( 'Plugin activated successfully.', 'rebar' ), 'success', 5500 );

			} else {

				/** Render "Activation failed" message. */
				UI::get_instance()->render_snackbar( esc_html__( 'Invalid purchase code.', 'rebar' ), 'error', 5500 );

			}

		}

	}

	/**
	 * Main SettingsFields Instance.
	 *
	 * Insures that only one instance of SettingsFields exists in memory at any one time.
	 *
	 * @static
	 * @return SettingsFields
	 * @since 1.0.0
	 **/
	public static function get_instance() {
		if ( ! isset( self::$instance ) && ! ( self::$instance instanceof SettingsFields ) ) {
			self::$instance = new SettingsFields;
		}

		return self::$instance;
	}

} // End Class SettingsFields.
